/* arch/arm/mach-s3c2410/tomtomgo-devs.h
 *
 * Definitions of TomTom GO platform devices.
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 * Author: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/config.h>
#include <linux/device.h>
#include <asm/hardware.h>
#include <asm/arch/tomtomgo-map.h>
#include <asm/arch/tomtomgo-irq.h>

/* TomTom GO IDE device */
//#if defined CONFIG_BARCELONA_IDE || defined CONFIG_BARCELONA_IDE_MODULE
static struct resource tomtomgo_ide_resource[] = {
	[0] = {
		.start = TOMTOMGO_PA_IDE,
		.end   = TOMTOMGO_PA_IDE + TOMTOMGO_SZ_IDE,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = TOMTOMGO_IRQ_IDE,
		.end   = TOMTOMGO_IRQ_IDE,
		.flags = IORESOURCE_IRQ,
	},
};

struct platform_device tomtomgo_device_ide = {
	.name          = "tomtomgo-ide",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_ide_resource),
	.resource      = tomtomgo_ide_resource,
};

EXPORT_SYMBOL(tomtomgo_device_ide);
//#endif /* CONFIG_BARCELONA_IDE || CONFIG_BARCELONA_IDE_MODULE */

/* TomTom GO hardware detection device */
#if defined CONFIG_BARCELONA_HW || defined CONFIG_BARCELONA_HW_MODULE
struct platform_device tomtomgo_device_hw = {
	.name          = "tomtomgo-hw",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_hw);
#endif /* CONFIG_BARCELONA_HW || CONFIG_BARCELONA_HW_MODULE */

/* TomTom GO ADC device */
#if defined CONFIG_BARCELONA_ADC || defined CONFIG_BARCELONA_ADC_MODULE
static struct resource tomtomgo_adc_resource[] = {
	[0] = {
		.start = S3C2410_PA_ADC,
		.end   = S3C2410_PA_ADC + S3C24XX_SZ_ADC,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_TC,
		.end   = IRQ_ADC,
		.flags = IORESOURCE_IRQ,
	},
};

struct platform_device tomtomgo_device_adc = {
	.name          = "tomtomgo-adc",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_adc_resource),
	.resource      = tomtomgo_adc_resource,
};

EXPORT_SYMBOL(tomtomgo_device_adc);
#endif /* CONFIG_BARCELONA_ADC || CONFIG_BARCELONA_ADC_MODULE */

/* TomTom GO accelerometer device */
#if defined CONFIG_BARCELONA_ACC || defined CONFIG_BARCELONA_ACC_MODULE
struct platform_device tomtomgo_device_acc = {
	.name          = "tomtomgo-acc",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_acc);
#endif /* CONFIG_BARCELONA_ACC || CONFIG_BARCELONA_ACC_MODULE */

/* TomTom GO battery device */
#if defined CONFIG_BARCELONA_BAT || defined CONFIG_BARCELONA_BAT_MODULE
struct platform_device tomtomgo_device_bat = {
	.name          = "tomtomgo-bat",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_bat);
#endif /* CONFIG_BARCELONA_BAT || CONFIG_BARCELONA_BAT_MODULE */

/* TomTom GO GPIO device */
#if defined CONFIG_BARCELONA_GPIO || defined CONFIG_BARCELONA_GPIO_MODULE
struct platform_device tomtomgo_device_gpio = {
	.name          = "tomtomgo-gpio",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_gpio);
#endif /* CONFIG_BARCELONA_GPIO || CONFIG_BARCELONA_GPIO_MODULE */

/* TomTom GO GPS device */
#if defined CONFIG_BARCELONA_GPS || defined CONFIG_BARCELONA_GPS_MODULE
static struct resource tomtomgo_gps_resource[] = {
	[0] = {
		.start = TOMTOMGO_IRQ_GPS_PPS,
		.end   = TOMTOMGO_IRQ_GPS_PPS,
		.flags = IORESOURCE_IRQ,
	},
};

struct platform_device tomtomgo_device_gps = {
	.name          = "tomtomgo-gps",
	.id            = -1,
	.num_resources = ARRAY_SIZE(tomtomgo_gps_resource),
	.resource      = tomtomgo_gps_resource,
};

EXPORT_SYMBOL(tomtomgo_device_gps);
#endif /* CONFIG_BARCELONA_GPS || CONFIG_BARCELONA_GPS_MODULE */

/* TomTom GO PWM device */
#if defined CONFIG_BARCELONA_PWM || defined CONFIG_BARCELONA_PWM_MODULE
struct platform_device tomtomgo_device_pwm = {
	.name          = "tomtomgo-pwm",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_pwm);
#endif /* CONFIG_BARCELONA_PWM || CONFIG_BARCELONA_PWM_MODULE */

/* TomTom GO remote control device */
#if defined CONFIG_BARCELONA_RC || defined CONFIG_BARCELONA_RC_MODULE
struct platform_device tomtomgo_device_rc = {
	.name          = "tomtomgo-rc",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_rc);
#endif /* CONFIG_BARCELONA_RC || CONFIG_BARCELONA_RC_MODULE */

/* TomTom GO storage device */
#if defined CONFIG_BARCELONA_SD || defined CONFIG_BARCELONA_SD_MODULE
struct platform_device tomtomgo_device_sd = {
	.name          = "tomtomgo-sd",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_sd);
#endif /* CONFIG_BARCELONA_SD || CONFIG_BARCELONA_SD_MODULE */

/* TomTom GO touchscreen */
#if defined CONFIG_BARCELONA_TS || defined CONFIG_BARCELONA_TS_MODULE
struct platform_device tomtomgo_device_ts = {
	.name          = "tomtomgo-ts",
	.id            = -1,
};

EXPORT_SYMBOL(tomtomgo_device_ts);
#endif /* CONFIG_BARCELONA_TS || CONFIG_BARCELONA_TS_MODULE */

/* EOF */
