/* arch/arm/mach-s3c2410/tomtomgo-ioexp.c
 *
 * Implementation of the IO expander driver.
 *
 * Copyright (C) 2006 TomTom BV <http://www.tomtom.com/>
 * Author: Jeroen Taverne <jeroen.taverne@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifdef __KERNEL__
#include <asm/uaccess.h>
#include <barcelona/gopins.h>
#endif /* __KERNEL__ */

#ifdef __BOOTLOADER__
#include "compiler.h"
#include "irqsave.h"
#include "Timer.h"
#include "gopins.h"
#define EXPORT_SYMBOL(x)
#endif  /* __BOOTLOADER__ */

#include "tomtomgo-iic.h"

#define IOEXP_ADDRESS 	0x40
#define IOEXP_IODIR 	0x00
#define IOEXP_IPOL  	0x01
#define IOEXP_GPINTEN 	0x02
#define IOEXP_DEFVAL 	0x03
#define IOEXP_INTCON 	0x04
#define IOEXP_IOCON 	0x05
#define IOEXP_GPPU 		0x06
#define IOEXP_INTF 		0x07
#define IOEXP_INTCAP 	0x08
#define IOEXP_GPIO 		0x09
#define IOEXP_OLAT 		0x0a

#define IOEXP_INPUT_MASK ~(1 << 2)
#define IOEXP_PULLUP_MASK ~((1 << 5) | (1 << 2))

#ifdef __KERNEL__
static spinlock_t ioexp_lock = SPIN_LOCK_UNLOCKED;
#endif

unsigned int IOEXP_InputOutput(unsigned outputValue,int force)
{
	static unsigned int result = 0xff;
	static int expanderDetected = 0;
	static int olddockint = 0; 
	static int dockint;
	static unsigned int timeTickReached = 0;
	static unsigned int timeTickReset	  = 0;
#ifdef __BOOTLOADER__
	extern unsigned int timeTick;
#endif
#ifdef __KERNEL__
	unsigned long flags;	
	#define timeTick (jiffies * 5)
#endif

#ifdef __KERNEL__
	spin_lock_irqsave(&ioexp_lock, flags);
#endif

	IO_Activate(DOCK_PWREN);
	
	IO_SetInput(SW_SDA);
	
	dockint = IO_GetInput(DOCK_INT);
	
	// Debounce dockint
	if (dockint != olddockint)
	{
		timeTickReached = timeTick + 500;
	}

	if ((force) || (timeTick > timeTickReached))
	{
		if ((force) || (dockint != expanderDetected))
		{
			expanderDetected = 0;
			if (IO_GetInput(SW_SDA))
			{
				IOIIC_Start();
				if (IOIIC_Transmit(IOEXP_ADDRESS)) 
				{
					expanderDetected = 1; // any of &= will reset expanderDetected
					expanderDetected &= IOIIC_Transmit(IOEXP_IODIR);		// Start with first register
					expanderDetected &= IOIIC_Transmit(IOEXP_INPUT_MASK);	// IODIR	Set input pins
					expanderDetected &= IOIIC_Transmit(0x00);				// IPOL		No pin inversion
					expanderDetected &= IOIIC_Transmit(IOEXP_INPUT_MASK);	// GPINTEN	Set interrupt on inputs
					expanderDetected &= IOIIC_Transmit(0x00);				// DEFVAL	Reset default value
					expanderDetected &= IOIIC_Transmit(0x00);				// INTCON	Compare with last pin value
					expanderDetected &= IOIIC_Transmit(0x00);				// IOCON	Active low interrupt (DOCK_INT will be low when no change is detected)
					expanderDetected &= IOIIC_Transmit(IOEXP_PULLUP_MASK);	// GPPU		Enable pullups for input pins
					expanderDetected &= IOIIC_Transmit(0x00);				// INTF		Read only register
					expanderDetected &= IOIIC_Transmit(0x00);				// INTCAP	Read only register
					expanderDetected &= IOIIC_Transmit(outputValue);		// GPIO		Send output data to GPIO register
					IOIIC_Stop();
					IOIIC_Start();
					expanderDetected &= IOIIC_Transmit(IOEXP_ADDRESS);		// Send chip address
					expanderDetected &= IOIIC_Transmit(IOEXP_GPIO);			// GPIO	Select GPIO register
					IOIIC_Stop();
					IOIIC_Start();
					expanderDetected &= IOIIC_Transmit(IOEXP_ADDRESS+1);	// Send chip address
					
					result = IOIIC_Receive(0);			// Receive GPIO data
					result |= (1 << 9);					// Simulate dock detect signal
					
					if (!expanderDetected) {
						// IO_Deactivate(DOCK_PWREN);
						timeTickReset   = timeTick + 500;
					}
				}
			}
			IOIIC_Stop();
		}

		dockint = IO_GetInput(DOCK_INT);

		// Check if IO expander is still connected
		if (!dockint)
		{
			expanderDetected = 0;
			result = 0xff;
		}

		timeTickReached = timeTick + 500;
	}

	// Check USB and desk dock if IO expander isn't connected
	if (!expanderDetected)
	{
		if (!IO_GetInput(SW_SDA))
		{
			if (dockint) result = 0xff | (1 << 10) | (1 << 8); else result = 0xff | (1 << 10);
		} else result = 0xff;
	}

	olddockint = dockint;

#ifdef __KERNEL__
	spin_unlock_irqrestore(&ioexp_lock, flags);
#endif

	return result;
}
