/* arch/arm/mach-s3c2410/tomtomgo-type.c
 *
 * Definitions for different TomTom GO types.
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 * Authors:
 * Jeroen Taverne <jeroen.taverne@tomtom.com>
 * Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifdef __KERNEL__
#include <linux/kernel.h>		/* for general stuff (e.g. printk) */
#include <linux/module.h>		/* for EXPORT_SYMBOL */
#include <linux/delay.h>		/* for mdelay */
#include <asm/system.h>			/* for local_irq_save/restore */
#include <asm/arch/map.h>		/* for S3C2410_VA_GPIO */
#include <barcelona/gopins.h>
#include <barcelona/gotype.h>
#include "pm.h"
#define GP_BASE		S3C24XX_VA_GPIO
#define DBG(fmt, arg...) printk(KERN_DEBUG "%s: " fmt, __func__ ,##arg)
#define INF(fmt, arg...) printk(KERN_INFO fmt ,##arg)
#endif /* __KERNEL__ */

#ifdef __BOOTLOADER__
#define GP_BASE		0x56000000
#define DBG(x)
#define INF(x,y,z)
#include "compiler.h"
#include "irqsave.h"
#include "Timer.h"
#include "gopins.h"
#include "gotype.h"
#include "poweroff.h"
#include <string.h>
#define EXPORT_SYMBOL(x)
#endif  /* __BOOTLOADER__ */

#include "tomtomgo-ioexp.h"

#define rGSTATUS1  (*(volatile unsigned *)(GP_BASE + 0xb0))

#define rGP_CON(p) (*(volatile unsigned*)(GP_BASE + ((p) << 4) + 0x0))
#define rGP_DAT(p) (*(volatile unsigned*)(GP_BASE + ((p) << 4) + 0x4))
#define rGP_UP(p)  (*(volatile unsigned*)(GP_BASE + ((p) << 4) + 0x8))

#ifdef __BOOTLOADER__
#define rEXTINT(p) (*(volatile unsigned *)(GP_BASE + ((p) << 4) + 0x88))
#endif

/* Get pin definition macros */
#include "tomtomgo-iopins.h"

#ifdef CONFIG_SMDK2440_BOARD
#include "tomtomgo-iosmdk2440.h"
#else
#include "tomtomgo-ioclassic.h"
#include "tomtomgo-iom100.h"
#include "tomtomgo-iom300.h"
#include "tomtomgo-iom500.h"
#include "tomtomgo-ioatlanta.h"
#include "tomtomgo-iobilbao.h"
#include "tomtomgo-ioglasgow.h"
#include "tomtomgo-iovalencia.h"
#include "tomtomgo-ioaberdeen.h"
#endif
#include "tomtomgo-ionopullup.h"
#include "tomtomgo-iosuspend.h"

#ifdef __BOOTLOADER__
static void IO_SetInterruptType(signed intNr,int intType)
{
	unsigned long flags;

	local_irq_save(flags);

	rEXTINT(intNr >> 3) &= ~(7 << ((intNr & 7) << 2));
	rEXTINT(intNr >> 3) |= (intType << ((intNr & 7) << 2));

	local_irq_restore(flags);
}
#endif

#ifdef __KERNEL__
#define IO_SetInterruptType set_irq_type
#endif

#ifdef __BOOTLOADER__
#define IRQ_EINT0  0
#define IRQ_EINT1  1
#define IRQ_EINT2  2
#define IRQ_EINT3  3
#define IRQ_EINT4  4
#define IRQ_EINT5  5
#define IRQ_EINT6  6
#define IRQ_EINT7  7
#define IRQ_EINT8  8
#define IRQ_EINT9  9
#define IRQ_EINT10 10
#define IRQ_EINT11 11
#define IRQ_EINT12 12
#define IRQ_EINT13 13
#define IRQ_EINT14 14
#define IRQ_EINT15 15
#define IRQ_EINT16 16
#define IRQ_EINT17 17
#define IRQ_EINT18 18
#define IRQ_EINT19 19
#define IRQ_EINT20 20
#define IRQ_EINT21 21
#define IRQ_EINT22 22
#define IRQ_EINT23 23

#define IRQT_LOW      0 
#define IRQT_HIGH     1 
#define IRQT_FALLING  2
#define IRQT_RISING   4 
#define IRQT_BOTHEDGE 6 
#endif /* __BOOTLOADER__ */

// Use an array so the defines can change without problems
static const signed irqList[] =
{
	IRQ_EINT0,
	IRQ_EINT1,
	IRQ_EINT2,
	IRQ_EINT3,
	IRQ_EINT4,
	IRQ_EINT5,
	IRQ_EINT6,
	IRQ_EINT7,
	IRQ_EINT8,
	IRQ_EINT9,
	IRQ_EINT10,
	IRQ_EINT11,
	IRQ_EINT12,
	IRQ_EINT13,
	IRQ_EINT14,
	IRQ_EINT15,
	IRQ_EINT16,
	IRQ_EINT17,
	IRQ_EINT18,
	IRQ_EINT19,
	IRQ_EINT20,
	IRQ_EINT21,
	IRQ_EINT22,
	IRQ_EINT23
};

/* Currently detected GO type */
struct gotype gotype_current;
EXPORT_SYMBOL(gotype_current);

/* Data from and to extended dock */
static unsigned int dockInputData = 0xff;
static unsigned int dockOutputData = 0x00;

int IO_Init(void)
{
	unsigned id;

	DBG("Starting detection\n");

#ifdef CONFIG_SMDK2440_BOARD
	id = GOTYPE_SMDK2440;
#else /* CONFIG_SMDK2440_BOARD */
	/* Read hardware type */
	switch (rGSTATUS1 & 0xfffffff0)
	{
		case 0x32440aa0:
			// Detect Valencia or ...
			IOP_SetInput(PIN_GPB5);
			IOP_SetInput(PIN_GPB6);
			mdelay(1);
			id = (IOP_GetInput(PIN_GPB6) * GOTYPE_ID1_HI) | (IOP_GetInput(PIN_GPB5) * GOTYPE_ID0_HI);
			id |= GOTYPE_SAMSUNG_TFT;
			id |= GOTYPE_S3C2442;
			break;
		case 0x32440000:
			// Detect M300, M500, Atlanta or Aberdeen 
			IOP_SetInput(PIN_GPB8);
			mdelay(1);
			if (IOP_GetInput(PIN_GPB8)) 
			{
				IOP_SetInput(PIN_GPC2);
				IOP_SetInput(PIN_GPC3);
				mdelay(1);
				id = (IOP_GetInput(PIN_GPC3) * GOTYPE_ID1_HI) | (IOP_GetInput(PIN_GPC2) * GOTYPE_ID0_HI);
				id |= GOTYPE_NEC_TFT;
			} 
			else 
			{
				IOP_SetInput(PIN_GPB5);
				IOP_SetInput(PIN_GPB6);
				IOP_SetInput(PIN_GPB7);
				mdelay(1);
				id = (IOP_GetInput(PIN_GPB7) * GOTYPE_ID2_HI) | (IOP_GetInput(PIN_GPB6) * GOTYPE_ID1_HI) | (IOP_GetInput(PIN_GPB5) * GOTYPE_ID0_HI);
				id |= GOTYPE_SAMSUNG_TFT;
			}
			id |= GOTYPE_S3C2440;
			break;
		case 0x32410000:
		default:
			// detect Classic or M100
			IOP_SetInput(PIN_GPC2);
			IOP_SetInput(PIN_GPC3);
			mdelay(1);
			id = (IOP_GetInput(PIN_GPC3) * GOTYPE_ID1_HI) | (IOP_GetInput(PIN_GPC2) * GOTYPE_ID0_HI);
			id |= GOTYPE_NEC_TFT;
			id |= GOTYPE_S3C2410;
			break;
	}

	/* Read sub ID from power board connected to Atlanta */
	if (id == GOTYPE_ATLANTA) 
	{
		IOP_SetInput(PIN_GPJ3);
		IOP_SetInput(PIN_GPJ5);
		mdelay(1);
		id |= (IOP_GetInput(PIN_GPJ5) * GOTYPE_ID4_HI) | (IOP_GetInput(PIN_GPJ3) * GOTYPE_ID3_HI);
	}
#endif /* CONFIG_SMDK2440_BOARD */
	
	/* Clear structure before init */
	memset(&gotype_current, 0, sizeof gotype_current);

	/* Init according to hardware type */
	switch (id) 
	{
#ifdef SUPPORT_CLASSIC
	case GOTYPE_CLASSIC:
		IO_InitClassic();
		break;
#endif
#ifdef SUPPORT_M100
	case GOTYPE_M100:
		IO_InitM100();
		break;
#endif
#ifdef SUPPORT_M300
	case GOTYPE_M300:
		IO_InitM300();
		break;
#endif
#ifdef SUPPORT_M500
	case GOTYPE_M500:
		IO_InitM500();
		break;
#endif
#ifdef SUPPORT_BILBAO
	case GOTYPE_BILBAO:
		IO_InitBilbao();
		break;
#endif
#ifdef SUPPORT_GLASGOW
	case GOTYPE_GLASGOW:
		IO_InitGlasgow();
		break;
#endif
#ifdef SUPPORT_VALENCIA
	case GOTYPE_VALENCIA:
		IO_InitValencia();
		break;
#endif
#ifdef SUPPORT_ABERDEEN
	case GOTYPE_ABERDEEN:
		IO_InitAberdeen();
		break;
#endif
#ifdef SUPPORT_EDINBURGH
	case GOTYPE_EDINBURGH:
		IO_InitEdinburgh();
		break;
#endif
#ifdef SUPPORT_SMDK2440
	case GOTYPE_SMDK2440:
		IO_InitSMDK2440();
		break;
#endif
	default:
		// Flash the backlight to show ID error
		IOP_Activate(PIN_GPB1 | PIN_INVERTED);
		while (1)
		{
			IOP_Activate(PIN_GPB0);
			mdelay(200);
			IOP_Deactivate(PIN_GPB0);
			mdelay(200);
		}
	}

	// For some reason Samsung changed the programmable pullups to programmable pulldowns in the S3C2442, so disable all pulldowns for all used pins
	if (IO_HavePullDowns())
	{
		gopin_t *pin;

		for (pin = &gotype_current.pins.FIRST; pin <= &gotype_current.pins.LAST; ++pin)
		{
			if ((*pin) && (!((*pin) & PIN_PULLDOWN))) *pin |= PIN_NO_PULLUP;
		}
	}	

	// Check something if SD and harddisk are both possible on the same hardware
	if ((IO_HaveSdCard()) == (IO_HaveHarddisk()))
	{
#if 1
		// Use flexcable ID to check if SD slot is available
		IO_SetInput(FLEX_ID1);
		IO_SetInput(FLEX_ID2);
		mdelay(1);
		if ((IO_GetInput(FLEX_ID1)) && (!IO_GetInput(FLEX_ID2))) IO_NoHarddiskDetected(); else IO_HarddiskDetected();
#else
		// Use SD insertion signal to check if SD slot is available
		IO_SetInput(CD_SD);
		mdelay(1);
		if (IO_GetInput(CD_SD)) IO_NoHarddiskDetected(); else IO_HarddiskDetected();
#endif
	}
	
	IO_InitNoPullUp();
	IO_InitSuspend();
	IO_InitDockUART();

	INF("Detected type %d, name %s\n", gotype_current.id, gotype_current.name);
	return 0;
}
EXPORT_SYMBOL(IO_Init);

void IO_NoHarddiskDetected(void)
{
#ifdef SUPPORT_VALENCIA
	if (IO_GetModelId() == GOTYPE_VALENCIA) IO_InitValencia500();
#endif
}
EXPORT_SYMBOL (IO_NoHarddiskDetected);

void IO_HarddiskDetected(void)
{
#ifdef SUPPORT_VALENCIA
	if (IO_GetModelId() == GOTYPE_VALENCIA) IO_InitValencia700();
#endif
}
EXPORT_SYMBOL (IO_HarddiskDetected);

void IO_InitDockUART(void)
{
	IO_Activate(DOCK_PWREN);
	IO_SetFunction(CTS_DOCK);
	IO_SetFunction(RTS_DOCK);
	IO_SetFunction(TXD_DOCK);
	IO_SetFunction(RXD_DOCK);
}
EXPORT_SYMBOL(IO_InitDockUART);

void IO_ExitDockUART(void)
{
	IO_Suspend(CTS_DOCK);
	IO_Suspend(RTS_DOCK);
	IO_Suspend(TXD_DOCK);
	IO_Suspend(RXD_DOCK);
	IO_Suspend(DOCK_PWREN);
}
EXPORT_SYMBOL(IO_ExitDockUART);

void IO_PowerOff(void)
{
	gopin_t *pin;
	unsigned char used[512];
#ifdef __BOOTLOADER__
	int i;
#endif
	
	memset(used,0,512);

	for (pin = &gotype_current.pins.FIRST; pin <= &gotype_current.pins.LAST; ++pin)
	{
		IOP_Suspend(*pin);
		if (*pin) used[*pin & 0x1ff] = 1;
	}

#ifdef __BOOTLOADER__
	for (i = PIN_GPB0;i <= PIN_GPB15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPC0;i <= PIN_GPC15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPD0;i <= PIN_GPD15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPE0;i <= PIN_GPE15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPF0;i <= PIN_GPF15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPG0;i <= PIN_GPG15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPH0;i <= PIN_GPH15;i++) if (!used[i]) IOP_Deactivate(i);
	for (i = PIN_GPJ0;i <= PIN_GPJ15;i++) if (!used[i]) IOP_Deactivate(i);
#endif
}
EXPORT_SYMBOL(IO_PowerOff);

void IO_Update(int force)
{
	if (IO_HaveIoExpander())
	{
		/* Force update on external IO expander */
		dockInputData = IOEXP_InputOutput(dockOutputData,force);
	}
}
EXPORT_SYMBOL(IO_Update);

inline static void IOP_SetBit(gopin_t pin, unsigned value)
{
	unsigned pinNr;
	unsigned portNr;
	unsigned long flags;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	if (portNr == PORT_GPIIC)
	{
		if (value)
			dockOutputData |= (1 << pinNr);
		else
			dockOutputData &= ~(1 << pinNr);
		return;
	}

	local_irq_save(flags);

	if (portNr == PORT_GPA)
	{
		// Set config to output
		rGP_CON(portNr) &= ~(1 << pinNr);
	}
	else
	{
		// Set config to output
		rGP_CON(portNr) = (rGP_CON(portNr) & ~(3 << (pinNr*2))) | (1 << (pinNr*2));
		// Disable pullup
		rGP_UP(portNr) |= (1 << pinNr);
	}

	// Set bit
	if (value)
		rGP_DAT(portNr) |= (1 << pinNr);
	else
		rGP_DAT(portNr) &= ~(1 << pinNr);

	local_irq_restore(flags);
}

void IOP_GeneratePWM(gopin_t pin)
{
	register int a, b, c;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return;

	for (a=0;a<10;a++)
	{
		for (b=0;b<40;b++)
		{
			for (c=0;c<a;c++) IOP_Activate(pin);
			for (c=a;c<10;c++) IOP_Deactivate(pin);
		}
	}
}
EXPORT_SYMBOL(IOP_GeneratePWM);

void IOP_Activate(gopin_t pin)
{
	if (PIN_IS_INVERTED(pin))
	{
		IOP_SetBit(pin, 0);
	}
	else
	{
		IOP_SetBit(pin, 1);
	}
}
EXPORT_SYMBOL(IOP_Activate);

void IOP_Deactivate(gopin_t pin)
{
	if (PIN_IS_FLOATING(pin))
	{
		IOP_SetInput(pin);
	}
	else if (PIN_IS_INVERTED(pin))
	{
		IOP_SetBit(pin, 1);
	}
	else
	{
		IOP_SetBit(pin, 0);
	}
}
EXPORT_SYMBOL(IOP_Deactivate);

void IOP_Suspend(gopin_t pin)
{
	if (pin & PIN_INPUT_ON_SUSPEND)
	{
		IOP_SetInput(pin);
	}
	else
	if (pin & PIN_ACTIVATE_ON_SUSPEND)
	{
		IOP_Activate(pin);
	}
	else
	{
		IOP_Deactivate(pin);
	}
}
EXPORT_SYMBOL(IOP_Suspend);

void IOP_SetFunction(gopin_t pin)
{
	unsigned pinNr;
	unsigned portNr;
	unsigned long flags;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	if (portNr == PORT_GPIIC)
	{
		return;
	}

	local_irq_save(flags);

	if (portNr == PORT_GPA)
	{
		// Set config to function
		rGP_CON(portNr) |= (1 << pinNr);
	}
	else
	{
		// Set config to function
		rGP_CON(portNr) = (rGP_CON(portNr) & ~(3 << (pinNr*2))) | (2 << (pinNr*2));
		// Disable pullup
		if (pin & PIN_PULLDOWN) {
			rGP_UP(portNr) &= ~(1 << pinNr);
		} else {
			rGP_UP(portNr) |= (1 << pinNr);
		}
	}

	local_irq_restore(flags);
}
EXPORT_SYMBOL(IOP_SetFunction);

void IOP_SetFunction2(gopin_t pin)
{
	unsigned pinNr;
	unsigned portNr;
	unsigned long flags;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	if (portNr == PORT_GPIIC)
	{
		return;
	}

	local_irq_save(flags);

	if (portNr == PORT_GPA)
	{
		// Set config to function
		rGP_CON(portNr) |= (1 << pinNr);
	}
	else
	{
		// Set config to function
		rGP_CON(portNr) = (rGP_CON(portNr) & ~(3 << (pinNr*2))) | (3 << (pinNr*2));
		// Disable pullup
		if (pin & PIN_PULLDOWN) {
			rGP_UP(portNr) &= ~(1 << pinNr);
		} else {
			rGP_UP(portNr) |= (1 << pinNr);
		}
	}

	local_irq_restore(flags);
}
EXPORT_SYMBOL(IOP_SetFunction2);

signed IOP_GetInterruptNumber(gopin_t pin)
{
	unsigned pinNr;
	unsigned portNr;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return -1;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	switch (portNr)
	{
	case PORT_GPF:
		return irqList[pinNr];
	case PORT_GPG:
		return irqList[pinNr + 8];
	default:
		return -1;
	}
}
EXPORT_SYMBOL(IOP_GetInterruptNumber);

static signed IOP_SetInterrupt(gopin_t pin)
{
	unsigned pinNr;
	unsigned portNr;
	unsigned long flags;
	signed intNr;

	intNr = IOP_GetInterruptNumber(pin);
	if (intNr < 0) return -1;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	switch (portNr)
	{
	case PORT_GPF:
		intNr = irqList[pinNr];
		break;
	case PORT_GPG:
		intNr = irqList[pinNr + 8];
		break;
	default:
		return -1;
	}

	local_irq_save(flags);

	if (portNr)
	{
		// Set config to interrupt
		rGP_CON(portNr) = (rGP_CON(portNr) & ~(3 << (pinNr*2))) | (2 << (pinNr*2));
		// Enable pullup for interrupt pin
		if (PIN_IS_NO_PULLUP(pin))
			rGP_UP(portNr) |= (1 << pinNr);
		else
			rGP_UP(portNr) &= ~(1 << pinNr);
	}

	local_irq_restore(flags);

	return intNr;
}

signed IOP_SetInterruptOnActivation(gopin_t pin)
{
	signed intNr;

	intNr = IOP_SetInterrupt(pin);
	if (intNr >= 0) IO_SetInterruptType(intNr, PIN_IS_INVERTED(pin) ? IRQT_FALLING : IRQT_RISING);

	return intNr;
}
EXPORT_SYMBOL(IOP_SetInterruptOnActivation);

signed IOP_SetInterruptOnDeactivation(gopin_t pin)
{
	signed intNr;

	intNr = IOP_SetInterrupt(pin);
	if (intNr >= 0) IO_SetInterruptType(intNr, PIN_IS_INVERTED(pin) ? IRQT_RISING : IRQT_FALLING);

	return intNr;
}
EXPORT_SYMBOL(IOP_SetInterruptOnDeactivation);

signed IOP_SetInterruptOnActivated(gopin_t pin)
{
	signed intNr;

	intNr = IOP_SetInterrupt(pin);
	if (intNr >= 0) IO_SetInterruptType(intNr, PIN_IS_INVERTED(pin) ? IRQT_LOW : IRQT_HIGH);

	return intNr;
}
EXPORT_SYMBOL(IOP_SetInterruptOnActivated);

signed IOP_SetInterruptOnDeactivated(gopin_t pin)
{
	signed intNr;

	intNr = IOP_SetInterrupt(pin);
	if (intNr >= 0) IO_SetInterruptType(intNr, PIN_IS_INVERTED(pin) ? IRQT_HIGH : IRQT_LOW);

	return intNr;
}
EXPORT_SYMBOL(IOP_SetInterruptOnDeactivated);

signed IOP_SetInterruptOnToggle(gopin_t pin)
{
	signed intNr;

	intNr = IOP_SetInterrupt(pin);
	if (intNr >= 0) IO_SetInterruptType(intNr, IRQT_BOTHEDGE);

	return intNr;
}
EXPORT_SYMBOL(IOP_SetInterruptOnToggle);

void IOP_SetInput(gopin_t pin)
{
	unsigned pinNr = pin & PIN_MASK;
	unsigned portNr;
	unsigned long flags;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	if (portNr == PORT_GPIIC)
	{
		return;
	}

	local_irq_save(flags);

	if (portNr == PORT_GPA)
	{
		return;
	}
	else
	{
		// Set config to input
		rGP_CON(portNr) = (rGP_CON(portNr) & ~(3 << (pinNr*2))) | (0 << (pinNr*2));
		// Enable pullup for input pin
		if (PIN_IS_NO_PULLUP(pin))
			rGP_UP(portNr) |= (1 << pinNr);
		else
			rGP_UP(portNr) &= ~(1 << pinNr);
	}

	local_irq_restore(flags);
}
EXPORT_SYMBOL(IOP_SetInput);

int IOP_GetInput(gopin_t pin)
{
	unsigned pinNr;
	unsigned portNr;

	// Don't do anything if pin isn't used
	if (PIN_IS_NOT_USED(pin))
		return 0;

	portNr = GET_PORTNR(pin);
	pinNr = GET_PINNR(pin);

	if (portNr == PORT_GPIIC)
	{
		if (dockInputData & (1 << pinNr))
		{
			return PIN_IS_INVERTED(pin) ? 0 : 1;
		}
		else
		{
			return PIN_IS_INVERTED(pin) ? 1 : 0;
		}
	}
	else
	if (rGP_DAT(portNr) & (1 << pinNr))
	{
		return PIN_IS_INVERTED(pin) ? 0 : 1;
	}
	else
	{
		return PIN_IS_INVERTED(pin) ? 1 : 0;
	}
}
EXPORT_SYMBOL(IOP_GetInput);

int IO_GetDockState(void)
{
 	if (IO_GetInput(DOCK_SENSE))
		return GODOCK_WINDSCREEN;
	else
		if (IO_GetInput(DOCK_PRO_SENSE))
			return GODOCK_PERMANENT;
		else
			if (IO_GetInput(DOCK_DESK_SENSE))
				return GODOCK_DESK;
			else
				return GODOCK_NONE;
}
EXPORT_SYMBOL(IO_GetDockState);

/* EOF */
