#!/bin/sh -e

# Default settings
verfile="tomtom.ver"
defcfg="config-default"
curcfg=".config"
update="n"
rebuild="n"
clean="n"
makeflags=""
makedefs="V=0"
makejobs="-j 4"

# Environment for building
export ARCH="arm"
export CROSS_COMPILE="arm-linux-"

die() {
  echo "$0: ERROR: $1"
  exit 1
}

usage() {
  echo "Usage: $0 {update|rebuild|clean|silent|verbose|single}"
  exit 1
}

get_version() {
  source ${verfile}
  if test -z "${TTVER}"; then
    die "Invalid version in ${verfile}!"
  fi

  if test "${update}" = "y"; then
    echo "**** Checking if you edited any sources..."
    if ! cvs -Qz9 diff > /dev/null 2>&1; then
      die "You need to commit your changes first."
    fi
    echo "**** Checking if your sources are up to date..."
    if ! cvs -Qnz9 commit > /dev/null 2>&1; then
      die "You need to update your sources first."
    fi
    TTVER=$((TTVER+1))
    sed -i "s/^TTVER=.*$/TTVER=${TTVER}/" ${verfile}
    echo "**** Committing new build number ${TTVER}..."
    cvs -Qz9 commit -m "New kernel build ${TTVER}."
    echo "**** Tagging with new build number ${TTVER}..."
    cvs -Qz9 tag build-tt${TTVER}
  fi
}

make_clean() {
  echo "**** Cleaning..."
  nice make ${makeflags} ${makedefs} distclean
}

make_config() {
  echo "**** Configuring with ${defcfg}..."
  cp -p "${defcfg}" "${curcfg}"
  nice make ${makeflags} ${makedefs} silentoldconfig
}

make_build() {
  echo "**** Building..."
  nice make ${makeflags} ${makejobs} ${makedefs}
}

# Main starts here
while test -n "$1"; do
  case "$1" in
  update)
    update="y"
    ;;
  rebuild)
    rebuild="y"
    ;;
  clean)
    clean="y"
    ;;
  silent)
    makeflags="-s"
    makedefs="V=0"
    ;;
  verbose)
    makeflags=""
    makedefs="V=1"
    ;;
  single)
    makejobs=""
    ;;
  config-*)
    if test ! -f "$1"; then
      die "$1 does not exist!"
    fi
    if test -f "${curcfg}" && ! rm -f "${curcfg}"; then
      die "Unable to remove ${curcfg}!"
    fi
    defcfg="$1"
    ;;
  *)
    usage
    ;;
  esac
  shift
done

get_version

# Set up our local version suffix
echo "-tt${TTVER}" > localversion

echo "**** Processing kernel #${TTVER}..."

if test "${rebuild}" = "y"; then
  make_clean
  make_config
elif test "${clean}" = "y"; then
  make_clean
  echo "**** Successfully cleaned kernel #${TTVER}."
  exit 0
elif test ! -f "${curcfg}" -o "${defcfg}" -nt "${curcfg}"; then
  make_config
fi

make_build

echo "**** Successfully built kernel #${TTVER}:"
ls -l arch/arm/boot/zImage

# EOF
