/* include/barcelona/gopins.h
 *
 * one line to give the program's name and an idea of what it does.
 *
 * Copyright (C) 2005 TomTom BV <http://www.tomtom.com/>
 * Authors: Dimitry Andric <dimitry.andric@tomtom.com>
 *          Jeroen Taverne <jeroen.taverne@tomtom.com>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __INCLUDE_BARCELONA_GOPINS_H
#define __INCLUDE_BARCELONA_GOPINS_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#if defined (__KERNEL__) || (__BOOTLOADER__)

typedef unsigned short gopin_t;

struct gopins {
	/* Just to find the address of the first pin */	
	gopin_t FIRST;

	/* SD Card Interface */
	gopin_t SD_PWR_ON;
	gopin_t SDCLK;
	gopin_t SDCMD;
	gopin_t SDDATA0;
	gopin_t SDDATA1;
	gopin_t SDDATA2;
	gopin_t SDDATA3;
	gopin_t WP_SD;
	gopin_t CD_SD;

	/* GPS interface */
	gopin_t GPS_RESET;
	gopin_t GPS_ON;
	gopin_t GPS_1PPS;
	gopin_t GPS_REPRO;
	gopin_t TXD_GPS;
	gopin_t RXD_GPS;

	/* Harddisk interface */
	gopin_t HDD_LED;
	gopin_t HDD_PWR_ON;
	gopin_t HDD_RST;
	gopin_t HDD_BUF_EN;
	gopin_t HDD_IRQ;
	gopin_t HDD_DRQ;
	gopin_t HDD_DACK;
	gopin_t HDD_CS;

	/* Power management */
	gopin_t ON_OFF;
	gopin_t IGNITION;
	gopin_t ACPWR;
	gopin_t PWR_RST;
	gopin_t CHARGEFAULT;
	gopin_t CHARGE_OUT;
	gopin_t CHARGING;
	gopin_t AIN4_PWR;
	gopin_t BATT_TEMP_OVER;

	/* Docking */
	gopin_t SW_SCL;
	gopin_t SW_SDA;
	gopin_t DOCK_INT;
	gopin_t DOCK_PWREN;
	gopin_t DOCK_SENSE;
	gopin_t DOCK_DESK_SENSE;
	gopin_t DOCK_PRO_SENSE;
	gopin_t LIGHTS_DETECT;
	gopin_t CTS_DOCK;
	gopin_t RTS_DOCK;
	gopin_t TXD_DOCK;
	gopin_t RXD_DOCK;
	gopin_t RXD_DOCK_INT;
	gopin_t LINEIN_DETECT;

	/* Remote */
	gopin_t FSK_FFS;
	gopin_t FSK_IRQ;
	gopin_t SPICLK;
	gopin_t SPIMSI;
	gopin_t SPIMSO;
	gopin_t FSK_EN;
	gopin_t FSK_CLKOUT;

	/* Accelerometer */
	gopin_t ACC_PWR_ON;
	gopin_t ACC_IIC_SDA;
	gopin_t ACC_IIC_SCL;
	gopin_t GYRO_EN;

	/* Sound */
	gopin_t MIC_SW;
	gopin_t AMP_ON;
	gopin_t DAC_PWR_ON;
	gopin_t MUTE_EXT;
	gopin_t MUTE_INT;
	gopin_t HEADPHONE_DETECT;
	gopin_t EXTMIC_DETECT;
	gopin_t I2SSDO;
	gopin_t CDCLK;
	gopin_t CDCLK_12MHZ;
	gopin_t I2SSCLK;
	gopin_t I2SLRCK;
	gopin_t I2SSDI;
	gopin_t L3CLOCK;
	gopin_t L3MODE;
	gopin_t L3DATA;

	/* Bluetooth */
	gopin_t TXD_BT;
	gopin_t RXD_BT;
	gopin_t RTS_BT;
	gopin_t CTS_BT;
	gopin_t BT_RESET;
	gopin_t BT_MODE;
	gopin_t BT_CLKOUT;

	/* LCD */
	gopin_t VD0;
	gopin_t VD1;
	gopin_t VD2;
	gopin_t VD3;
	gopin_t VD4;
	gopin_t VD5;
	gopin_t VD6;
	gopin_t VD7;
	gopin_t VD8;
	gopin_t VD9;
	gopin_t VD10;
	gopin_t VD11;
	gopin_t VD12;
	gopin_t VD13;
	gopin_t VD14;
	gopin_t VD15;
	gopin_t VD16;
	gopin_t VD17;
	gopin_t VD18;
	gopin_t VD19;
	gopin_t VD20;
	gopin_t VD21;
	gopin_t VD22;
	gopin_t VD23;
	gopin_t VDEN;
	gopin_t VCLK;
	gopin_t LCD_VCC_PWREN;
	gopin_t LCD_BIAS_PWREN;
	gopin_t LCD_OEN;
	gopin_t HSYNC;
	gopin_t VSYNC;
	gopin_t BACKLIGHT_EN;
	gopin_t BACKLIGHT_PWM;
	gopin_t LCD_CS;          
	gopin_t LCD_SCL;         
	gopin_t LCD_SDI;         
	gopin_t LCD_RESET;
	gopin_t LCD_PON;       

	/* USB */
	gopin_t USB_HOST_DETECT;
	gopin_t USB_PULL_EN;
	gopin_t USB_PWR_EN;
	gopin_t USB_RST;
	gopin_t USB_SUSPEND;
	gopin_t USB_EJECT;
	gopin_t USB_CLKOUT;
	gopin_t USB_IIC_SDA;
	gopin_t USB_IIC_SCL;
	gopin_t USB_DACK;
	gopin_t USB_DREQ;
	gopin_t USB_IRQ;
	gopin_t USB_VBUS;
	gopin_t USB_RESET;
	gopin_t USB_HP;
	gopin_t USB_SUSPEND_OUT;

	/* Touchscreen */
	gopin_t XMON;
	gopin_t XPON;
	gopin_t YMON;
	gopin_t YPON;
	gopin_t TSDOWN;
	
	/* CPU */
	gopin_t LOW_CORE;
	
	/* Flex cable */
	gopin_t FLEX_ID1;
	gopin_t FLEX_ID2;

	/* Light sensor */	
	gopin_t LX_EN;
	
	/* Unused pins that should be set low at suspend */
	gopin_t UNUSED_SDA;
	gopin_t UNUSED_SCL;

	/* Just to find the address of the last pin */	
	gopin_t LAST;
};

struct gotype {
	const char *name;
	const char *shortname;
	const char *familyname;
	const char *projectname;
	const char *requiredbootloader;
	unsigned short id;
	unsigned short usbproductid;
	unsigned int cpufreq;
	unsigned int bluetoothfreq;
	unsigned int bootloaderfreq;
	unsigned char tsxchannel;    	
	unsigned char tsychannel;
	unsigned char cputype;
	unsigned char bluetooth;
	unsigned char usbbluetooth;
	unsigned char lowbutton;
	unsigned char handsfree;
	unsigned char headsetgateway;
	unsigned char a2dp;
	unsigned char battery;
	unsigned char sdcard;
	unsigned char harddisk;
	unsigned char tsfets;
	unsigned char remote;
	unsigned char docking;
	unsigned char tfttype;
	unsigned char gpstype;
	unsigned char bttype;
	unsigned char headphoneconnector;
	unsigned char speaker;
	unsigned char codectype;
	unsigned char codecmaster;
	unsigned char acctype;
	unsigned char usbslavetype;
	unsigned char memsize;
	unsigned char lightsensor;
	unsigned char gyro;
	unsigned char ohciports;
	unsigned char harddisktiming;
	unsigned char loquendo;
	unsigned char regulatorforcodec;
	unsigned char ioexpander;
	unsigned char rxddetect;
	unsigned char pulldowns;
	unsigned char usbvbuspower;
	unsigned char tftsoftstart;
	struct gopins pins;
};

int IO_Init(void);

extern struct gotype gotype_current;

void IO_NoHarddiskDetected(void);
void IO_HarddiskDetected(void);
void IO_InitDockUART(void);
void IO_ExitDockUART(void);
void IO_PowerOff(void);
void IO_ForceUpdate(void);
void IOP_Activate(gopin_t pin);
void IOP_Deactivate(gopin_t pin);
void IOP_GeneratePWM(gopin_t pin);
void IOP_Suspend(gopin_t pin);
void IOP_SetFunction(gopin_t pin);
void IOP_SetFunction2(gopin_t pin);
void IO_Update(int force);
int IO_GetDockState(void);
signed IOP_SetInterruptOnActivation(gopin_t pin);
signed IOP_SetInterruptOnDeactivation(gopin_t pin);
signed IOP_SetInterruptOnActivated(gopin_t pin);
signed IOP_SetInterruptOnDeactivated(gopin_t pin);
signed IOP_SetInterruptOnToggle(gopin_t pin);
signed IOP_GetInterruptNumber(gopin_t pin);

void IOP_SetInput(gopin_t pin);
int IOP_GetInput(gopin_t pin);

#define IO_GetModelId()				(gotype_current.id)
#define IO_GetModelName()			(gotype_current.name)
#define IO_GetFamilyName()			(gotype_current.familyname)
#define IO_GetProjectName()			(gotype_current.projectname)
#define IO_GetShortName()			(gotype_current.shortname)
#define IO_GetCpuType()				(gotype_current.cputype)
#define IO_GetCpuFreq()				(gotype_current.cpufreq)
#define IO_HaveBluetooth()			(gotype_current.bluetooth)
#define IO_GetBluetoothFreq()		(gotype_current.bluetoothfreq)
#define IO_HaveUsbBluetooth()		(gotype_current.usbbluetooth)
#define IO_HaveLoweredButton()		(gotype_current.lowbutton)
#define IO_HaveHandsfree()			(gotype_current.handsfree)
#define IO_HaveHeadsetGateway()		(gotype_current.headsetgateway)
#define IO_HaveA2DP()				(gotype_current.a2dp)
#define IO_HaveBattery()			(gotype_current.battery)
#define IO_HaveSdCard()				(gotype_current.sdcard)
#define IO_HaveHarddisk()			(gotype_current.harddisk)
#define IO_HaveTsFets()				(gotype_current.tsfets)
#define IO_HaveRemote()				(gotype_current.remote)
#define IO_HaveDocking()			(gotype_current.docking)
#define IO_GetTftType()				(gotype_current.tfttype)
#define IO_GetTsXChannel()			(gotype_current.tsxchannel)
#define IO_GetTsYChannel()			(gotype_current.tsychannel)
#define IO_GetGpsType()				(gotype_current.gpstype)
#define IO_HaveHeadphoneConnector()	(gotype_current.headphoneconnector)
#define IO_HaveSpeaker()			(gotype_current.speaker)
#define IO_GetCodecType()			(gotype_current.codectype)
#define IO_GetCodecMaster()			(gotype_current.codecmaster)
#define IO_GetAccType()				(gotype_current.acctype)
#define IO_GetUSBSlaveType()		(gotype_current.usbslavetype)
#define IO_GetUSBProductID()		(gotype_current.usbproductid)
#define IO_GetMemSize()				(gotype_current.memsize)
#define IO_HaveLightSensor()		(gotype_current.lightsensor)
#define IO_HaveGyro()				(gotype_current.gyro)
#define IO_UsbOhciPortMask()		(gotype_current.ohciports)
#define IO_GetHarddiskTiming()      (gotype_current.harddisktiming)
#define IO_HaveLoquendo()           (gotype_current.loquendo)
#define IO_HaveRegulatorForCodec()  (gotype_current.regulatorforcodec)
#define IO_HaveIoExpander()         (gotype_current.ioexpander)
#define IO_HaveRxdDetect()          (gotype_current.rxddetect)
#define IO_HavePullDowns()          (gotype_current.pulldowns)
#define IO_GetRequiredBootloader()  (gotype_current.requiredbootloader)
#define IO_HaveTftSoftStart()       (gotype_current.tftsoftstart)

#define IO_Pin(x)	(gotype_current.pins.x)

#define IO_Activate(x)						IOP_Activate(IO_Pin(x))
#define IO_Deactivate(x)					IOP_Deactivate(IO_Pin(x))
#define IO_GeneratePWM(x)					IOP_GeneratePWM(IO_Pin(x))
#define IO_Suspend(x)						IOP_Suspend(IO_Pin(x))
#define IO_SetFunction(x)					IOP_SetFunction(IO_Pin(x))
#define IO_SetFunction2(x)					IOP_SetFunction2(IO_Pin(x))
#define IO_SetInterruptOnActivation(x)		IOP_SetInterruptOnActivation(IO_Pin(x))
#define IO_SetInterruptOnDeactivation(x)	IOP_SetInterruptOnDeactivation(IO_Pin(x))
#define IO_SetInterruptOnActivated(x)		IOP_SetInterruptOnActivated(IO_Pin(x))
#define IO_SetInterruptOnDeactivated(x)		IOP_SetInterruptOnDeactivated(IO_Pin(x))
#define IO_SetInterruptOnToggle(x)			IOP_SetInterruptOnToggle(IO_Pin(x))     
#define IO_SetInput(x)						IOP_SetInput(IO_Pin(x))
#define IO_GetInput(x)						IOP_GetInput(IO_Pin(x))
#define IO_GetInterruptNumber(x)			IOP_GetInterruptNumber(IO_Pin(x))

/* Special type for SMDK2440, not to be used from user-space */
#define GOTYPE_SMDK2440			0x0fffffff

/* GO CPU types */
#define GOCPU_UNDEFINED			0	/* Undefined */
#define GOCPU_S3C2410			1	/* Samsung S3C2410 */
#define GOCPU_S3C2440			2	/* Samsung S3C2440 */
#define GOCPU_S3C2442			3	/* Samsung S3C2442 */
#define GOCPU_S3C2413			4	/* Samsung S3C2413 */

/* GO TFT types */
#define GOTFT_UNDEFINED			0	/* Undefined */
#define GOTFT_SMDK2440			1	/* SMDK 2440 */
#define GOTFT_NEC_NL2432HC22	2	/* NEC NL2432HC22-22B */
#define GOTFT_SAMSUNG_LTV350	3	/* Samsung LTV350QV */
#define GOTFT_SAMSUNG_LTP400	4	/* Samsung LTP400WQ */

/* GO GPS types */
#define GOGPS_UNDEFINED			0	/* Undefined */
#define GOGPS_SIRF1				1	/* Sirf 1 */
#define GOGPS_SIRF2				2	/* Sirf 2 */
#define GOGPS_SIRF3				3	/* Sirf 3 */

/* GO BT types */
#define GOBT_NONE				0	/* No bluetooth */
#define GOBT_BC1				1	/* Bluecore 1 */
#define GOBT_BC2				2	/* Bluecore 2 */
#define GOBT_BC3				3	/* Bluecore 3 */
#define GOBT_BC4				4	/* Bluecore 4 */

/* GO Codec types */
#define GOCODEC_NONE			0	/* No codec */
#define GOCODEC_WM8711			1	/* Wolfson WM8711 */
#define GOCODEC_WM8971			2	/* Wolfson WM8971 */

/* GO Acc types */
#define GOACC_NONE				0
#define GOACC_MXR2312			1
#define GOACC_MXR3999			2
#define GOACC_MXM9301			3
#define GOACC_MXR9500			4

/* GO USB slave types */
#define GOUSB_S3C24XX			0
#define GOUSB_TUSB6250			1
#define GOUSB_NET2272			2

/* GO dock types */
#define GODOCK_NONE				0
#define GODOCK_WINDSCREEN		1
#define GODOCK_PERMANENT		2
#define GODOCK_DESK				3

#else /* (__KERNEL__) || (__BOOTLOADER__) */
#error "Never include this header from user space."
#endif /* (__KERNEL__) || (__BOOTLOADER__) */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __INCLUDE_BARCELONA_GOPINS_H */

/* EOF */
